#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <fcntl.h>

/*
 * Sarah Jelinek
 * University of Colorado, Colorado Springs
 * Computer Science Masters Project
 * 10/01/03
 * Copyright 2003
 */

/*
 * 3.1 IDIP Application Message Header
 * p.4
 */
#define	IDIP_APP_VERS_IDIP	0x0010
#define	IDIP_APP_VERS_CIDF	0x0100

/*
 * 3.1 IDIP Application Message Header
 * p.6 Table 1
 */
#define	IDIP_APP_CLASS_TRACE	0x0020
#define	IDIP_APP_CLASS_DCUNDO	0x002a
#define	IDIP_APP_CLASS_DCDO	0x002e

#define IDIP_LIMIT_RATE		0x1
#define IDIP_DROP		0x2
#define IDIP_RESTORE_RATE	0x3

/*
 * 3.1 IDIP Application Message Header
 * p.7 Figure 3
 */
struct idip_app_orig_addr {
	uint32_t	addr;
	uint32_t	ident;
	uint32_t	boottime;
	uint32_t	pad;
};

/*
 * 3.1 IDIP Application Message Header
 * p.4 Figure 2
 */
struct idip_app_msg_hdr {
	uint8_t				version;
	uint8_t				class_id;
	uint32_t			length;
	uint32_t			timestamp;
	uint32_t			thread_id;
	struct idip_app_orig_addr	orig_addr;
	uint8_t				flags;
	uint8_t				pad[3];
	uint32_t			timediff;
};

/*
 * 3.2.1.2 Results Header
 * p.9 Figure 6
 */
struct idip_app_res_hdr {
	uint8_t				severity;
	uint8_t				certainty;
	uint16_t			outcome;
	uint32_t			attack_code;
	uint32_t			attack_start;
	uint32_t			attack_end;
	uint8_t				observer[256];
	uint8_t				attack_name[256];
	uint8_t				detector_specific[256];
};

/*
 * 3.2.1.1 Trace Header
 * p.9 Figure 5
 */
struct idip_app_trace_hdr {
	uint32_t			event_count;
	uint32_t			examine_from;
	uint32_t			examine_until;
	uint32_t			original_thread_id;
};

/*
 * 3.2.1.3 Requested Action Header
 * p.10-12 Figures 7-12
 */
struct idip_app_req_action_hdr {
	/* audit header, figure 8 p. 10 */
	uint32_t			audit_from;
	uint32_t			audit_until;
	/* block header, figure 9 p. 11 */
	uint32_t			block_from;
	uint32_t			block_until;
	/* allow header, figure 11 p. 11 */
	uint32_t			allow_from;
	uint32_t			allow_until;
	/* request header, figure 12 p. 12 */
	uint32_t			request_action;
	uint32_t			request_from;
	uint32_t			request_until;
	/* limit header, figure 10 p. 11 */
	uint32_t			limit_from;
	uint32_t			limit_until;
	uint32_t			limit_rate;
	uint32_t			burst_rate;
	uint32_t			limit_action;
};

/*
 * 3.2.1.4 Extensions
 * p.12-16 Figures 13-21
 */
struct idip_app_ext_hdr {
	/* extension header, same as app. msg header */
	struct idip_app_msg_hdr		ext_hdr;
	/* refer header, figure 14 p. 13 */
	struct idip_app_orig_addr	orig_idip_addr;
	uint32_t			thread_id;
	uint32_t			rcv_from;
	/* response headers, figure 15 p. 13 */
	/* trace response header, figure 16 p. 14 */
	uint32_t			trace_event_cnt;
	uint32_t			trace_examine_from;
	uint32_t			trace_examine_until;
	uint32_t			trace_thread_id;
	uint32_t			trace_io_in;
	uint32_t			trace_io_out;
	/* audit response header, figure 17 p. 14 */
	uint32_t			audit_from;
	uint32_t			audit_until;
	uint32_t			audit_io_in;
	uint32_t			audit_io_out;
	/* block response header, figure 18 p. 14 */
	uint32_t			block_from;
	uint32_t			block_until;
	uint32_t			block_io_in;
	uint32_t			block_io_out;
	/* allow response header, figure 21 p. 15 */
	uint32_t			allow_from;
	uint32_t			allow_until;
	uint32_t			allow_io_in;
	uint32_t			allow_io_out;
	/* limit response header, figure 19 p. 15 */
	uint32_t			limit_from;
	uint32_t			limit_until;
	uint32_t			limit_rate;
	uint32_t			limit_burst_rate;
	uint32_t			limit_action;
	uint32_t			limit_io_in;
	uint32_t			limit_io_out;
	/* request response header, figure 20 p. 15 */
	uint32_t			request_action;
	uint32_t			request_from;
	uint32_t			request_until;
};

/*
 * 3.2.1.5 Trace Data
 * p. 17 Figure 23
 */
#define	CIDD_PROTOCOL_ICMP	0x01;
#define	CIDD_PROTOCOL_TCP	0x06;
#define	CIDD_PROTOCOL_UDP	0x17;

struct idip_app_cidd {
	uint8_t				ip_protocol;
	uint8_t				protocol_mask;
	uint32_t			ip_source;
	uint32_t			ip_source_mask;
	uint32_t			ip_dest;
	uint32_t			ip_dest_mask;
	uint8_t				port_ranges[10];
};

/*
 * 3.2.1.5 Trace Data
 * p.16 Figure 22
 */
struct idip_app_trace_data {
	struct idip_app_cidd		cidd;
	struct idip_app_cidd		xcidd;
	uint32_t			session_id;
	uint32_t			pid;
	uint32_t			username;
	uint32_t			hostname;
	uint32_t			groupname;
	uint32_t			procname;
	uint32_t			role;
	uint32_t			reqsecproto;
};

/*
 * 3.2.1 Trace/Report/Do message
 * p.8 Figure 4
 */
struct idip_app_trace_msg {
	struct idip_app_res_hdr		results_hdr;
	struct idip_app_trace_hdr	trace_hdr;
	struct idip_app_req_action_hdr	req_action_hdr;
	struct idip_app_ext_hdr		ext_hdr;
	struct idip_app_trace_data	data;
};

/*
 * 3.2.4 DC Undo/DC Undo Response Message
 * p.19-20 Figure 24
 */
struct idip_app_dc_undo_msg {
	struct idip_app_orig_addr	orig_idip_addr;
	uint32_t			thread_id;
	uint8_t				succeeded;
	uint8_t				pad[3];
};

/*
 * IDIP Application Message definition.
 */
typedef struct idip_app_msg {
	struct idip_app_msg_hdr		app_hdr;
	union {
	    struct idip_app_trace_msg	trace;
	    struct idip_app_dc_undo_msg	undo;
	} body;
	char			data[1024]; /* user defined field */
} idip_app_msg_t;   		
